// PyBOINCAPI - Python bindings to the BOINC API
//
// Copyright (C) 2009 Nicolás Alvarez, Jeremy Cowles
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <Python.h>
#include "boincmodule.h"
#include <diagnostics.h>
#include <boinc_api.h>

char* get_lib_name();

using namespace std;

void print_usage(const char* name)
{
    cerr << "Module name missing\n";
    cerr << "Usage: " << name << " MODULE LIBRARY [ADDITIONAL ARGS FOR MODULE]" << endl;
    cerr << endl;
    cerr << "    MODULE:  the name of a Python file to load" << endl;
    cerr << "    LIBRARY: a quoted list of one or more libraries" << endl << endl;
	cerr << "Example: " << name << " my_app.py \"python26.zip\" --other --args" << endl;
}

int main(int argc, const char* argv[])
{
	// sanity check on args
    if (argc < 3) {
        print_usage(argv[0]);
        return 1;
    }

    // let boinc know we are starting up and setup the diags
	// system to report a stack trace if we crash
	int ret = boinc_init();
	if (ret){
		cerr << "CRITICAL: boinc_init failed with error " << ret << endl;
		return 1;
	}
	boinc_init_diagnostics(BOINC_DIAG_DUMPCALLSTACKENABLED);


	pyboinc_app_name = (char*)argv[0];
	char* main_lib = (char*)argv[2];
	char* module = (char*)argv[1];
   
    // get the python library name 
    const int SIZE = 255;
    char libset[SIZE] = "PYTHONPATH=";
    strncat(libset, main_lib,SIZE);
	
    // add the specified library to sys.path via
    // the PYTHONPATH environment variable
#ifdef _WIN32
	// putenv is deprecated under windows, so use the oddly underscored one
    _putenv(libset); 
#else
	putenv(libset);
#endif

    // setup the program name and the python home
    // paths so that the interpreter will look for 
    // a local copy of the standard library in 
    // ./lib/python2.5 (where 2.5 is the version
    // of the header files used when compiling)
    Py_SetProgramName(pyboinc_app_name);
    Py_SetPythonHome("./");
    Py_NoSiteFlag = true;

    // Initialize the interpreter
    Py_Initialize();

	// pass args into the interpreter, but hide the executable
    PySys_SetArgv(argc-1, (char**)(argv+1));

	// add the boinc module
    Py_InitModule("boinc", BoincMethods);
	
	// startup.py loads a zip file as the main library instead of 
	// using the ./lib sub directory, it also sets up the sys.path
	// so that all common paths are added, similar to what site.py does
	// TODO: check for existance of startup.py to avoid seg-faults
	char* filename = "startup.py";
	PyObject* PyFileObject = PyFile_FromString(filename, "r");
	PyRun_AnyFile(PyFile_AsFile(PyFileObject), filename);
	Py_DECREF(PyFileObject);

	// Execute the module. Using PyRun_AnyFile so that __name__ == "__main__"
	// TODO: stat module file, it currently seg-faults when the file is missing
	PyFileObject = PyFile_FromString(module, "r");
	PyRun_AnyFile(PyFile_AsFile(PyFileObject), module);
	Py_DECREF(PyFileObject);

	// stop the interpreter and let boinc know we are done
    Py_Finalize();
	boinc_finish(0);
    return 0;
}

// returns a pointer to the library name 
// caller is responsible for deleting memory
char* get_lib_name(){
    const int SIZE = 13;
    const char* pyver = Py_GetVersion();
    char ver[3] = {pyver[0],pyver[2], '\0'};
    char* lib = new char[SIZE];
    strncat(lib, "python", SIZE);
    strncat(lib, ver, SIZE);
    strncat(lib, ".zip", SIZE);
    return lib;
}
